# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
SMS Database Stuff
"""

from __future__ import unicode_literals

try:
    from locsms.db import Session
except ImportError:
    Session = None

from rattail.config import ConfigExtension as BaseExtension


class ConfigExtension(BaseExtension):
    """
    Config extension for the ``rattail_locsms.db`` subpackage.  This extension
    is responsible for loading the available LOC SMS database engine(s), and
    configuring the :class:`pylocsms:pylocsms.db.Session` class with the
    default engine.  This extension expects to find something like the
    following in your config file:

    .. code-block:: ini

       [rattail_locsms.db]
       keys = default, host, other
       default.url = mssql://sms-dsn-default
       host.url = mssql://sms-dsn-host
       other.url = mssql://sms-dsn-other

    The result of this extension's processing is that the config object will
    get two new attributes:

    .. attribute:: rattail.config.RattailConfig.locsms_engines

       Dict of available LOC SMS database engines.  Keys of the dict are the
       same as found in the config file; values are the database engines.  Note
       that it is possible for this to be an empty dict.

    .. attribute:: rattail.config.RattailConfig.locsms_engine

       Default database engine; same as ``locsms_engines['default']``.  Note
       that it is possible for this value to be ``None``.
    """
    key = 'rattail_locsms.db'

    def configure(self, config):
        from rattail.db.config import get_engines

        if Session:
            engines = get_engines(config, section='rattail_locsms.db')
            config.locsms_engines = engines
            config.locsms_engine = engines.get('default')
            Session.configure(bind=config.locsms_engine)
